/******************************************************************************
FILE:				MModem.hh
LANGUAGE:			C++
SYSTEM:				None
USER-INTERFACE:		None
DESCRIPTION
	This class define an abstract Minitel modem module.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1993/12/21 <PJB> Creation.
	1994/01/02 <PJB> Added isConnected.
	1994/01/26 <PJB> Added MModemControler protocol.
	1994/01/29 <PJB> Tuned MModemControler protocol.
	1994/02/15 <PJB> Added processInput and processOutput.
	1994/02/22 <PJB> Added className method.
	1994/03/25 <PJB> Added parity inhibition option.
LEGAL
	Copyright Pascal J. Bourguignon 1993 - 1993
	All Rights Reserved.
******************************************************************************/
#ifndef __MModem__
#define __MModem__
#include <mnames.h>
#include BCINTERFACE_H
#include BCTYPES_H
#include MMODULE_HH
#include MTIMER_HH


class MModemControler
{
public:
////// senderRef is the outputRef of the sender.
// sent while connecting:
	METHOD(MModemControler,connectionFailed,(INT32 senderRef),void)
	METHOD(MModemControler,carrierDetected,(INT32 senderRef),void)
// sent once connected:
	METHOD(MModemControler,connected,(INT32 senderRef),void)
	METHOD(MModemControler,disconnected,(INT32 senderRef),void)
// returning to standard mode:
	METHOD(MModemControler,startTurnAround,(INT32 senderRef),void)
	METHOD(MModemControler,spurious,(INT32 senderRef),void)
// returning completed (any one):
	METHOD(MModemControler,turnedAround,(INT32 senderRef),void)
};//MModemControler



#ifdef OPTIMIZE_SPACE

	enum {
		MModem_State_Disconnected,
		MModem_State_Connecting1,
		MModem_State_Connecting2,
		MModem_State_Connected,
		MModem_State_Returning	
	};
	typedef CARD8			MModem_StateT;

	enum {
		MModem_Mode_Standard,	// = send at 75bps, receive at 1200bps.
		MModem_Mode_Slave,		// = send at 1200bps, receive at 75bps.
		MModem_Mode_Opposed		// = slave without returning.
	};
	typedef CARD8			MModem_ModeT;

	enum {
		MModem_Parity_Even,		// default
		MModem_Parity_Odd,
		MModem_Parity_None,
		MModem_Parity_Ignore
	};
	typedef CARD8			MModem_ParityT;	// cf note near setParity.

#else
	typedef enum {
		MModem_State_Disconnected,
		MModem_State_Connecting1,
		MModem_State_Connecting2,
		MModem_State_Connected,
		MModem_State_Returning	
	}						MModem_StateT;

	typedef enum {
		MModem_Mode_Standard,	// = send at 75bps, receive at 1200bps.
		MModem_Mode_Slave,		// = send at 1200bps, receive at 75bps.
		MModem_Mode_Opposed		// = slave without returning.
	}						MModem_ModeT;

	typedef enum {
		MModem_Parity_Even,		// default
		MModem_Parity_Odd,
		MModem_Parity_None,
		MModem_Parity_Ignore
	}						MModem_ParityT;	// cf note near setParity.
	
#endif




	PROCEDURE(MModem_SetParity,(MModem_ParityT parity,
				char* outbuf,const char* buffer,INT32 count),void)
	PROCEDURE(MModem_CheckParity,(MModem_ParityT parity,
				char* outbuf,const char* buffer,INT32 count),void)
	PROCEDURE(MModem_ClearParity,(char* buffer,INT32 count),void)


	typedef struct {
		CARD32				highBytesSent;		// received from Protocole.
		CARD32				highBytesReceived;	// transmited to Protocole.
		CARD32				bytesSent;			// transmited to serial line.
		CARD32				bytesReceived;		// received from serial line.
		CARD32				parityErrors;
		CARD32				frameErrors;	// SEE: serial line processed them.
	// ecp statistics:
		CARD32				correctCount;		// rem=0 & parErr=0
		CARD32				correctedCount;		// rem#0 & parErr=1 & corrected
		CARD32				uncorrectedCount;	// rem#0 & parErr=1 & can't.
		CARD32				uncorrectableCount;	// otherwise
		CARD32				invalidCount;		// byte 17 not 0.
		CARD32				nakCount;			// nak blocks sent.
		CARD32				validSynCount;		// valid Syn blocks.
		CARD32				invalidSynCount;	// invalid Syn blocks.
	}					MModem_StatisticsT;


// protected typedefs (client must not use):
#define MModem_EcpBlockSize		(17)
	typedef struct {
		BOOLEAN				active;
		CARD8				state;
		CARD8				count;						// bytes in block.
		CARD8				parityIndex;				// of first error.
		CARD8				parityErrors;
		CARD8				blockNumber;				// current block.
		CARD16				filler;		// block must be CARD32-aligned.
		char				block[MModem_EcpBlockSize];	// incoming block.
		MTimer*				nakTimer;
	}					MModem_EcpDataT;
	


#define MModem_SUPER MModule
class MModem:public MModem_SUPER
{
public:

	CONSTRUCTOR(MModem,(void))
	DESTRUCTOR(MModem,(void))
	
// MObject methods:

	METHOD(MModem,className,(void),const char*)

// MModule methods:

	METHOD(MModem,process,(const char* buffer,INT32 count,INT32 ref),void)
		/*
			Subclasses must either override this process() method, or the 
			protected processOutput/processInput methods below.
			If process() is overriden then the subclass may call processInput 
			to interpret incoming data (ECP, when active, or else check parity 
			and sendToOutput).
			
			process(buf,len,outputRef) or processOutput must send bytes to 
			the phone line. Bytes received from the phone line should be
			sendToOutput()'ed.
			
			process:
				PRE: state()=Connected.
				if ref = serial line ref then
					check and filter parity bits and sentToOutput
					else add parity bits and send to serial line.
			
		*/

	METHOD(MModem,sendToOutput,(const char* buffer,INT32 count),void)
		/*
			DO:			update statistics, then super::sentToOutput.
		*/
	
// MModem methods:

	METHOD(MModem,setControler,(MModemControler* nModemControler),void)
	METHOD(MModem,controler,(void),MModemControler*)


	METHOD(MModem,state,(void),MModem_StateT)
	METHOD(MModem,mode,(void),MModem_ModeT)
	
	METHOD(MModem,setMode,(MModem_ModeT newMode),void)
		// PRE: state()=Disconnected.


	METHOD(MModem,dialAndConnect,(const char* number),void)
		// PRE: state()=Disconnected.
		// not truly a Minitel1B function, but added for Hayes modems,
		// or emulation with a Minitel1B via its socket.
	METHOD(MModem,connect,(void),void)
		// PRE: state()=Disconnected.

	METHOD(MModem,disconnect,(void),void)
		// PRE: state()!=Disconnected.

	METHOD(MModem,startTurnAround,(void),void)
		// modem is in standard mode (master) and going to slave mode:
	
	// Error Correcting Procedure:
	
	METHOD(MModem,startECP,(void),void)
	METHOD(MModem,stopECP,(void),void)
	METHOD(MModem,isEcpActive,(void),BOOLEAN)
		// PRE: state()=Connected.
	

	// Statistics:
	
	METHOD(MModem,resetStatistics,(void),void)
		// POST:		the counters are reset.
		
	METHOD(MModem,getStatistics,(MModem_StatisticsT* statistics),void)
		// POST:		(*statistics) contains the current counters.

	METHOD(MModem,setParity,(MModem_ParityT nParity),void)
	METHOD(MModem,parity,(void),MModem_ParityT)
		/*
			NOTE: Yet another thing not in STUM1B, but some Videotex server
			      do not implement the even-parity rule, notably those
				  accessed directly via phone, or TRANSPAC. It seems like
				  the even-parity is enforced only by the Videotex PAD.
			NOTE: Even-parity is always enforced during ECP.
		*/
	
// protected:
	
	METHOD(MModem,modemTimerElapsed,(INT32 reference),void)
	
protected:
	MModemControler*	modemControler;
	MModem_StateT		mState;
	MModem_ModeT		mMode;
	MModem_ParityT		mParity;
	MModem_EcpDataT		ecp;
	MModem_StatisticsT	mStatistics;
	
	METHOD(MModem,processOutput,(const char* buffer,INT32 count),void)
		/*
			DO:			update statistics.
			NOTE:		Subclasses who override this method must
						call it in their version of processOutput.
		*/
	METHOD(MModem,processInput,(const char* buffer,INT32 count),void)
		/*
			Processes the bytes from the network. If ECP is active, then
			interpret them as ECP blocks (It implements the ECP protocol),
			otherwise, it merely sendToOutput.
			This method may be used by subclasses when they override the
			process() method to process data from the network.
		NOTE: No green bytes should go thru these processOutput and
			processInput methods (in the buffer/count block). If the 
			even-parity data flow from the videotex server is 
			encapsulated in an envelop, then the process method must be
			overriden to extract the videotex data and pass it to this
			processInput method. Either the process method or the
			processOutput method may encapsulate the data from the terminal to
			the videotex server, but the processOutput method must not be 
			passed encapsulated data.
		*/

};//MModem.


#endif //MModem.
